#######################################################################
#
# NCM component for mkgridmap daemon
#
# Charles Loomis <charles.loomis@cern.ch>
# Version 1.0.6 : 17/12/04 07:04
#
# ** Generated file : do not edit **
#
#######################################################################

package NCM::Component::mkgridmap;

use strict;
use NCM::Component;
use vars qw(@ISA $EC);
@ISA = qw(NCM::Component);
$EC=LC::Exception::Context->new->will_store_all;
use NCM::Check;

use EDG::WP4::CCM::Element;

use File::Copy;

local(*DTA);


##########################################################################
sub Configure($$@) {
##########################################################################
    
    my ($self, $config) = @_;

    # Define paths for convenience. 
    my $base = "/software/components/mkgridmap";
    my $vobase = "/system/vo";

    # Save the date.
    my $date = localtime();

    # Header of the file.
    my $contents = << "EOF"
##############################################################################
#
# File generated by ncm-mkgridmap on $date
#
##############################################################################

EOF
;

    # Header for the VO sections.
    $contents .= << "EOF"

##############################################################################
# EDG Virtual Organisations
# eg 'group ldap://grid-vo.cnaf.infn.it/ou=testbed1,o=infn,c=it .infngrid'
##############################################################################

EOF
;

    # Loop over all of the defined VOs to fill out this section. Looking at
    # the values: /system/vo/*/auth/*/(uri|user). 
    my $volist = $config->getElement("$vobase");
    while ($volist->hasNextElement()) {
        my $voentry = $volist->getNextElement();
        my $i1 = $voentry->getName();

        my $voname = $config->getValue("$vobase/$i1/name");
        $contents .= "# $voname\n";

        my $authlist = $config->getElement("$vobase/$i1/auth");
        while ($authlist->hasNextElement()) {
            my $authentry = $authlist->getNextElement();
            my $i2 = $authentry->getName();

            my $uri = $config->getValue("$vobase/$i1/auth/$i2/uri");
            my $user = $config->getValue("$vobase/$i1/auth/$i2/user");

            $contents .= "group $uri $user\n";
        }
    }

    # Header for the authorization groups section.
    $contents .= << "EOF"

#############################################################################
# List of auth URIs
# eg 'auth ldap://marianne.in2p3.fr/ou=People,o=testbed,dc=eu-datagrid,dc=org'
#
# If these are defined then users must be authorised in one of the following
# auth servers.
#############################################################################

EOF
;

    # Loop over all of the defined authorization URIs.  Looking at the
    # values: /software/components/*.
    if ($config->elementExists("$base/authURIs")) {
        my $authlist = $config->getElement("$base/authURIs");
        foreach ($authlist->getList()) {
            my $uri = $_->getValue();
            $contents .= "auth $uri\n";
        }
    }

    # Header for the authorization groups section.
    $contents .= << "EOF"

#############################################################################
# DEFAULT_LCLUSER: default_lcluser lcluser
# e.g. 'default_lcuser .'
#############################################################################

EOF
;

    # Default local user. Looking at the value:
    # /software/components/mkgridmap/lcuser.
    if ($config->elementExists("$base/lcuser")) {
        $contents .= "default_lcuser " . 
            $config->getValue("$base/lcuser") . "\n";
    }

    # Header for allow/deny patterns. 
    $contents .= << "EOF"

#############################################################################
# ALLOW and DENY: deny|allow pattern_to_match
# e.g. 'allow *INFN*'
#############################################################################

EOF
;

    # Default local user. Looking at the value:
    # /software/components/mkgridmap/lcuser.
    if ($config->elementExists("$base/allow")) {
        $contents .= "allow " . 
            $config->getValue("$base/allow") . "\n";
    }
    if ($config->elementExists("$base/deny")) {
        $contents .= "deny " . 
            $config->getValue("$base/deny") . "\n";
    }

    # Header for local gridmap file.
    $contents .= << "EOF"

#############################################################################
# Local grid-mapfile to import and overide all the above information.
# e.g. 'gmf_local /opt/edg/etc/grid-mapfile-local'
#############################################################################

EOF
;

    # Local gridmap file. Looking at the value:
    # /software/components/mkgridmap/gmflocal.
    if ($config->elementExists("$base/gmflocal")) {
        $contents .= "gmf_local " . 
            $config->getValue("$base/gmflocal") . "\n";
    }

    # File to modify (default is /opt/edg/etc/edg-mkgridmap.conf). 
    my $fname = $config->getValue("$base/mkgridmapconf");

    # Now just create the new configuration file.  Be careful to save
    # a backup of the previous file if necessary. 
    if ( ! -e $fname ) {
        
        # Configuration file doesn't exist yet.  Create it. 
        open CONF, ">$fname";
        print CONF $contents;
        close (CONF);
        $self->log("$fname created");
        
    } else {
        
        # Already exists. Make backup and create new file. 
        my $result = LC::Check::file( $fname,
                                      backup => ".old",
                                      contents => $contents,
                                      );
        $self->log("$fname updated") if $result;
    }


    # Do the same for the local gridmap file if necessary. 
    if ($config->elementExists("$base/gmflocal") &&
        $config->getValue("$base/overwrite")) {

        $fname = $config->getValue("$base/gmflocal");

        $contents = << "EOF"
###########################################################################
#
# File generated by ncm-mkgridmap on $date
# 
# A list of user mappings that will be imported into the grid-mapfile.
# 
###########################################################################

EOF
;

        # Loop over all of the defined authorization URIs.  Looking at the
        # values: /software/components/mkgridmap/locals/(cert|user).
        if ($config->elementExists("$base/locals")) {
            foreach ($config->getElement("$base/locals")->getList()) {
                my $index = $_->getName();
                my $cert = $config->getValue("$base/locals/$index/cert");
                my $user = $config->getValue("$base/locals/$index/user");
                $contents .= '"' . $cert . '" ' . $user . "\n";
            }
        }

        # Now just create the new configuration file.  Be careful to save
        # a backup of the previous file if necessary. 
        if ( ! -e $fname ) {
            
            # Configuration file doesn't exist yet.  Create it. 
            open ( CONF,">$fname" );
            print CONF $contents;
            close (CONF);
            $self->log("$fname created");
            
        } else {
            
            # Already exists. Make backup and create new file. 
            my $result = LC::Check::file( $fname,
                                          backup => ".old",
                                          contents => $contents,
                                          );
            $self->log("$fname updated") if $result;
        }
    }

    # Try regenerating the gridmap file. 
    if ($config->elementExists("$base/command")) {
	my $cmd = $config->getValue("$base/command");

	`$cmd`;
	$self->error("regeneration of the gridmap file failed") if ($?);
    }
    
    return 1;
}

1;      # Required for PERL modules
